<div id='top'/>
# AVOS: What How To and Not What Do

**`AVOS`** aka ***Pure Luck*** or ***Blind Faith***

*a guide to implementing functional atrocities too sublime to ignore* **or** *ramblings and preambles leading to recourse and discourse explaining the structural scaffold behind the purposive simple architecture*

---


1. Webcams Functions
    1. [Model Id List to Data](#ExtractWebcamsFields)
    1. [Expanding Models Data from Mysql](#ExpandWebcamsFields)
    1. [Related Webcams](#Related_Webcams)

2. Webcams Statistics
    1. [All Statistics](#AllStatistics)
    1. [Statistics Summary](#Webcams_Statistics)
    1. [Popular Models](#Popular_Models)
    1. [Popular Models Snapshots](#Popular_Models_Snapshots)
    1. [Trending Models](#Trending_Models)
    1. [Highest Popular Rank](#Highest_Popular_Rank)
    1. [Popular Categories](#Popular_Categories)
    1. [Popular Categories Snapshots](#Popular_Categories_Snapshots)
    1. [Trending Categories](#Trending_Categories)

3. [Users](#Users)
    1. [Saved Searches](#Saved_Searches)
    1. [Profile Viewing History](#Profile_Viewing_History)
    1. [Alerts](#Alerts)
4. [Posts](#Posts)


5.  Reference
    1. [Logging Requests](#Logging_Requests)
    1. [Date Constrained Template Variables](#Date_Constrained_Template_Variables)
    1. [Config Options](#Config_Options)


6. Webcams Add-Ons
    1. [SitesFilter + SitebarSitesFilter](#SitesFilter)
    2. [Promote Models | Events](#Promote)
   

7. [Blogs](#Blogs)
     1. [Add  Blogs](#Add_blogs)

8. Avos install Satelits Sites
    1. [Add New Satelit](#Add-Satelit)

---

## Webcams

<div id='ExtractWebcamsFields'/>
### Model Id List to Data

The function `extractWebcamsFields($webcamsIds)` will convert the list of `webcams` id's to an array of hashes containing the models info for display on a grid etc.

#### Example

    $models = extractWebcamsFields([12345, 12725]);

    [
        [
            [id] => 12345
            [addedOn] => 1631607911
            [updatedOn] => 1741005250
            [scrapedOn] => 1670274856
            [purged] => 0
            [nickname] => riley_sweety
            [externalId] => 0
            [name] => Riley
            [site] => Chaturbate
            [status] => Offline
            ...
        ],
        [
            [id] => 12725
            ...
        ]
    ]



---

<div id='ExpandWebcamsFields'/>
### Expand Mysql Webcams Data

The function `expandWebcamsFields($models)` takes a list of hashes containing webcams data extracted from mysql and expands/adds the following fields:

- imagePath
- siteShortName
- siteShortNameCaps
- site
- status
- gender

Additionally if age is 0 it is converted to the empty string and any data stored in `Redis` in the key `modelId/p` is merged.



---

<div id='Related_Webcams'/>
### Related Webcams

Fetches a specified number of webcams related to the current model identified by `$model` in the template or `getTemplateValue('model')` within external php code.

    getRelatedWebcams($cats=false, $count=8, $type='tags', $catstags=false, $samesite=false)

#### Parameters

- `$cats` is a csv of the list of cats and/or tags to check for related models, leave empty to default to all tags+cats on the site
- `$count` is how many related models to return
- `$type` indicates whether to first search `tags` or `categories`
- `$catstags` is optional, set to `true` if want to check categories filtered by tags
- `$samesite` is optional, set to `true` to ensure related models are on the same camsite as the current one

To use defaults simply use `setTemplateValue('related', getRelatedWebcams());`

#### Return Value

A list of models data is returned.

    [
        [
            [id] => 3099130
            [nickname] => em1ly_joy
            [site] => Chaturbate
            [status] => Freechat
            [addedOn] => 1741643103
            [age] => 20
            [hd] => 0
            [gender] => Female
            [imageUrl] => "https://thumb.live.mmcdn.com/ri/em1ly_joy.jpg"
            [imagePath] => "/photos/chaturbate/e/m/1/l/em1ly_joy/"
            [numImages] => 4
            [siteShortNameCaps] => Cb
        ],
        ...

#### Cookbook

A standard example to fetch a number of related webcams for a profile page:

    // create CSV of tags followed by categories
    $counts = getWebcamsCounts();

    // if set to false this is what $csv will expand to
    $csv = implode(',', array_merge(array_keys($counts['tags']), array_keys($counts['categories'])));

    // check to see if we have a [urls] rule that uses categories filtered by tags
    $useCatsTags = empty(getUrl('webcamsCategoryTag')) ? 0 : 1;

    // fetch related webcams and set as $related for the profile template
    setTemplateValue('related', getRelatedWebcams($csv, 20, 'tags', $useCatsTags));

Fetch one extra model from the same camsite for use in the profile page:

    $model2 = getRelatedWebcams(false, 1, 'tags', false, true)[0];
    setTemplateValue('model2', $model2);



---

## Webcams Statistics

<div id='Webcams_Statistics'/>
### Statistics Summary

The function `getStatsSummary()` returns an array containing 3 keys: 

- 'd' (day, last 24 hours online)
- 'm' (month, last 30 days)
- 'a' for all time data.

Each key contains a hash with these keys:

    [viewersTotal] => 330
    [viewersMax] => 48
    [viewersAvg] => 6
    [followersNew] => 704
    [followersMin] => 31477
    [followersMax] => 32181
    [rankMax] => 868
    [rankAvg] => 1541
    [onlineTime] => 371170
    [onlineDays] => 52
    [onlineDaily] => 7137
    [freechatTime] => 350825
    [freechatLongest] => 32041
    [freechatSessions] => 142
    [freechatAvg] => 2470
    [groupTime] => 11704
    [groupLongest] => 4740
    [groupSessions] => 22
    [groupAvg] => 532
    [privateTime] => 8641
    [privateLongest] => 2160
    [privateSessions] => 12
    [privateAvg] => 720

The data is cached for `statsCacheExpiry` seconds before needing to be recalculated, default is 8 hours. To pre-cache this data for all online models run the 'summary` cron. 
The initial run of this cron could take around an hour assuming approx 20k models online. Subsequent runs will only check/update the cache for models whose status has been
updated since the previous cron run.

#### config.ini

    [cron]
    domains/main = avos.mysite.com
    main/15minutes = summary

#### functions.php

    function webcamsStats()
    {
        setTemplateValue('stats', getStatsSummary());
    }



---

<div id='AllStatistics'/>
### All Statistics
The function `getAllStats($webcamsId)` returns all statistics for the specified model both recent data from Redis and all time data that has been purged to SQLite databases on the filesystem. It returns an `array()` with the following keys:

- 'h' (online status timeline by month)

    - eg: 12345/h/2501

- 'd' (summary of daily times in freechat etc)

    - eg: 12345/d/250210

- 'r' (ranks; followers and viewers)

    - eg: 12345/r/2412

    - e: epoch minutes since EPOCHSTART
    - r: rank
    - g: gender rank
    - v: viewers
    - f: followers

`[e]` is the timestamp in minutes after `EPOCHSTART` to reduce space as cron only runs every minute. to convert to epoch seconds use `$elt['e'] * 60 + EPOCHSTART`



---

<div id='Popular_Models'/>
### Popular Models

Stores the `popularModelsMax` models from the last `popularModelsDays` days, ranking is based on viewers. Run the `popular` cron
to cache the data.

Fetch the data using `getPopularWebcams($onlineOnly)`. default is `$onlineOnly == false` to return all models.

#### config.ini

    [webcams]
    ; these are the defaults if not set
    popularModelsMax = 1000
    popularModelsDays = 30

    [urls]
    webcamsPopular = [ none, /popular/ ]

#### functions.php

    function webcamsPopular()
    {
        setTemplateValue('models', getPopularWebcams());
    }



---

<div id='Popular_Models_Snapshots'/>
### Popular Models Snapshots

Every day a snapshot is saved for any model in the popular models list storing their max *viewers* and *followers* for that day. This is saved in redis as `prefix/popular/snapshots`

Data is stored for the last `popularModelsMaxViewersDays` days

Fetch the data using `getPopularWebcamsSnapshots($id=false, $day=false)`

#### Cookbook

##### config.ini

    [webcams]
    popularModelsMaxViewersDays = 30

##### Specific Model and Date

    \getPopularWebcamsSnapshots(98739, 250323);

    viewers => 8805
    followers => 1332202


##### Specific Model

    \getPopularWebcamsSnapshots(98739);

    250323 => [
        viewers => 8712
        followers => 1332202
    ],
    250324 => [
        viewers => 8805
        followers => 1333203
    ]


##### Specific Date

    \getPopularWebcamsSnapshots(false, 250323);

    1750741 => [
        viewers => 11179
        followers => 162215
    ],
    1891989 => [
        viewers => 129
        followers => 0
    ],
    ...

##### All Snapshots

    \getPopularWebcamsSnapshots();

    250323 => [
        1750741 => [
            viewers => 11179
            followers => 162215
        ],
        1891989 => [
            viewers => 129
            followers => 0
        ],
        ...
    250324 => [
        ...


---

<div id='Highest_Popular_Rank'/>
### Highest Popular Rank

If a model has ever ranked in the top `popularModelsMax` models list their best ranking and date is stored in redis under the key `siteprefix/models/modelId/p`

These values are automatically fetched from redis when displaying a profile as `bestRank` and `bestRankOn`

#### webcamsProfile.php

    <? if (!empty($model['bestRank'])): ?>
    Best Ranked <?= $model['bestRank'] ?> on <?= date("F j, Y", $model['bestRankOn']); ?></div>
    <? endif; ?>



---

<div id='Trending_Models'/>
### Trending Models

Iterates round the trendingModelsSnapshots snapshots of data, one per minute, which contains the number of viewers in each room. The deltas are calculated and added together to calculate 
a trending score, weighted towards more recent snapshots as `value * sqrt(snapshotNumber)`.

The returned list starts at rooms deemed to be most trending to the least. Default mode is absolute delta differences. Two other modes are available: `relative` which divides the value by viewers in the room and `average` which averages out the trending score per model from the two calculated lists.

#### config.ini

    [webcams]
    ; these are the defaults if not set
    trendingModelsSnapshots = 10
    trendingModelsMax = 1000

    [urls]
    ; type: absolute, relative, average
    webcamsTrending = [ none,   /trending/:type/ ]

#### functions.php

    function webcamsTrending()
    {
        $type = getUrl()['args']['type'];
        setTemplateValue('models', getTrendingWebcams($type));

        /*
        // default trending list based on absolute viewer deltas
        setTemplateValue('modelsa', getTrendingWebcams());
        setTemplateValue('modelsa', getTrendingWebcams('absolute'));

        // trending list based on the relative change in viewers
        setTemplateValue('modelsr', getTrendingWebcams('relative'));

        // average mode
        setTemplateValue('modelsa', getTrendingWebcams('average'));
        */
    }

#### webcamsTrending.php

    <? foreach ($models as $m): ?>
        <?= $m['nickname'] ?> trending rank is <?= $m['rank'] ?>
    <? endforeach; ?>



---

<div id="Popular_Categories"/>
### Popular Categories

Creates snapshots every minute for each category (categories, tags, sites) creating a rating score for each based on the number of online models and viewers optionally
weighted by `popularCategoriesRoomsWeight` and `popularCategoriesViewersWeight`. It reads from `prefix/webcams/counts` in redis and caches in `prefix/popular/categories`

`getPopularCategories()` returns the categories ordered by models, viewers and weightings. Calling with no argument returns data for categories, tags and sites. Alternatively 
passing in the type required returns only that data.

Runs at the end of the **categories** cron.

#### config.ini

    [webcams]
    popularCategoriesSnapshots = 30
    popularCategoriesRoomsWeight = 10
    popularCategoriesViewersWeight = 1

#### functions.php

    $sites = getPopularCategories('sites');

    [sc] => [
        [fullname] => StripChat
        [online] => 7154
        [viewers] => 187992
    ],
    [cb] => [
        [fullname] => Chaturbate
        [online] => 4418
        [viewers] => 223883
    ]
    ...



---

<div id="Popular_Categories_Snapshots"/>
### Popular Categories Snapshots

Every day a snapshot is saved for every category/tag/site max total *viewers* and *rooms* for that day. This is saved in redis as `prefix/popular/categories`

Data is stored for the last `popularCategoriesMaxViewersDays` days

Fetch the data using `getPopularCategoriesSnapshots($mode=false, $name=false)`

#### Cookbook

#### config.ini

    [webcams]
    popularCategoriesMaxViewersDays = 30

#### Specific type and name

    getPopularCategoriesSnapshots('categories', 'teens')

    250322 => [
        online => 344
        viewers => 130469
    ],
    250323 => [
        online => 344
        viewers => 130469
    ]

#### Specific type

    getPopularCategoriesSnapshots('tags')

    girls => [
        250322 => [
            online => 7829
            viewers => 322431
        ],
        250323 => [
            online => 7829
            viewers => 322431
        ]
    ]
    anal => [
        250322 => [
            online => 3206
            viewers => 118296
        ],
        250323 => [
            online => 3206
            viewers => 118296
        ]
    ]


#### All Snapshots

    getPopularCategoriesSnapshots('tags')

    categories => [
        test => [
            250322 => [
                online => 7843
                viewers => 328175
            ],
            250323 => [
                online => 7843
                viewers => 328175
            ]
        ],
        ...
        girls => [
            ...
    ],
    tags => [
        ...
    ],
    sites => [
        ...
    ]

#### Percentage Change Over Time for Viewers per Tag

    getPopularCategoriesSnapshots('tags');
    $change = [];

    foreach ($tags as $name => $stats) {
        $first = current($stats);
        $last = end($stats);
        $change[$name] = ($last['viewers'] - $first['viewers']) / $first['viewers'] * 100;
    }



---

<div id="Trending_Categories"/>
### Trending Categories

Calculates a score based on the delta between snapshots for each category based on the number of online models and viewers optionally weighted by 
`popularCategoriesRoomsWeight` and `popularCategoriesViewersWeight`

Shares the configuration and data with **Popular Categories** and the function `getTrendingCategories()` behaves the same as `getPopularCategories()`

---

<div id='Users'/>
## Users

The following work whether the user is logged in or not. When logged in data is stored in mysql and in the session if not.

An `array()` is returned from the users functions containing status and data which can automatically returned in json format if required for javascript use
(append `format=json` to a web request to a `[urls]` rule of type `users`)

    define('USERS_STATUS_OK',           1);
    define('USERS_STATUS_ERROR',        0);

    [
        [status] => 1
        [message] => 
        [error] => 0
        [args] => [
            [models] => []
    ]



---

<div id='Saved_Searches'/>
### Saved Searches

`saveSearch()` is automatically called when multiple webcams results are found. 
To display the searches  use `action=searches` in `[urls]` where the template (e.g: `usersSearches.php`) can iterate around `$searches`.

    [urls]
    usersSearches = [ users, /searches/, 'action=searches' ]

Use `action=clearSearches` to clear the searches:

    [urls]
    usersSearchesClear = [ users, /searches/clear/, 'action=clearSearches' ]

**usersSearchesClear.php**

    <? include("usersSearches.php");


#### Manually Calling Functions

You can also directly call `listUsersSearches()` and `clearUsersSearches()`

**config.ini**

    [urls]
    usersSearches = [ none, /searches/ ]

**functions.php**

    function usersSearches()
    {
        if (isset($_GET['clear'])) clearUsersSearches();
        setTemplateValue('searches', listUsersSearches()['args']['searches']);
    }



---

<div id='Profile_Viewing_History'/>
### Profile Viewing History

`addUsersHistory()` needs to be called in the `[urls]` rule for displaying a profile and `listUsersHistory()` will return a list of model id's that have been viewed.

#### config.ini

    [urls]
    usersHistory =             [ none,   /history/:page ]

#### functions.php

    function webcamsProfile()
    {           
        $model = getTemplateValue('model');
        addUsersHistory($model['id']);
    }

    function usersHistory()
    {
        setTemplateValue('models', listUsersHistory()['args']['models']);
    }



---

<div id="Alerts"/>

### Alerts

Users alerts are fetched using `listAlerts()` consisting of the following:

- logged on models from a users favorite list (for a logged in user)
- `usersAlertsNumPopular` online **popular** models
- all online **trending** models
- top `usersAlertsPerTag` online models from the top `usersAlertsNumTags` trending tags

#### config.ini

    [users]
    alertsNumPopular = 10
    alertsNumTags = 3
    alertsPerTag = 5

    [urls]
    alerts =      [ users, /alerts/,       'action=alerts' ]
    alertsDebug = [ users, /alerts-debug/, 'action=alerts&format=print' ]
    alertsAjax =  [ users, /alerts-json/,  'action=alerts&format=json' ]


---

<div id='Posts'/>
## Posts

Posts can be displayed individually via the variable `$post` given to the template or in multiples optionally filtered
by tag or keyword via either path or the `?kw=keyword` query arg in the usual way. 
The template `postsIndex.php` is given the variable `$posts` which is a list of hashes of posts data.

### config.ini

    [urls]
    postsIndex =               [ posts,     /pages/:page ]
    postsKeyword =             [ posts,     /pages/kw-:kw/:page ]
    postsTag =                 [ posts,     /pages/:tag/:page ]
    postsSingle =              [ post,      /:urlTitle.html ]

    ; following url will load the template post.php or postsSingle.php if postsSiski.php doesnt exist
    postsSiski =               [ post,      /i-really-love-siski, "urlTitle=i-love-siski-and-sosiki" ]

### postsIndex.php

    foreach ($posts as $p) {
        title: $p['title']
        link: encodeUrl('postsPage', $p)
        views: $p['views']
        body: $p['body']
        tags:
            foreach ($p['tags'] as $t) {
                tag name: $t
                tag link: encodeUrl('postsTag', ['tag' => $t])
            }
    }



---


## Reference

<div id="Logging_Requests"/>

### Logging Requests

Setting `logRequestsExpiry` in the `[site]` block will enable logging of requests in redis. Items in the queue are removed via the `logs` cron
after `logRequestsExpiry` hours. The is data is available in the admin by clicking on `Requests` in the sidebar.

Each item contains a hash of: timestamp, country, url path, `[urls]` type, `[urls]` name, decoded args and user agent.

    [
        [0] => 1742406080
        [1] => RO
        [2] => /
        [3] => webcamsProfile
        [4] => webcam
        [5] => [
            [site] => streamate
            [nickname] => isalondon
        ],
        [6] => "curl/7.64.1"
    ]

#### Cookbook

    // getRequests() returns data from newest to oldest
    $requests = \getRequests();

or

    // fetching directly from redis will return data oldest to newest
    $requests = \Avos\Redis\handle()->lRange(\Avos\Redis\getPrefix('requests'), 0, -1);




---

<div id="Date_Constrained_Template_Variables"/>

### Date Constrained Template Variables

It is possible to define variables within the `[templates]` block that are only set within a specified date range. This can be used to display
links or banners for a chosen promo period.

The `dateFormat` is defined as 1 or 2 digit month, 1 or 2 digit month/day or as a range defined as `dateFormat to dateFormat` or `dateFormat - dateFormat`

#### Syntax

    [templates]
    groupName:date = dateFormat
    groupName:variableName1 = value1
    groupName:variableName2 = value2

#### Date Formats

    12
    15/2
    12 to 3
    15/12 to 3/3
    12 - 3/3

#### Cookbook

    xmas:date = 10/12 to 1
    xmas:bannerImage = /image,png
    xmas:bannerLink = https://foo.com




---

<div id="Config_Options"/>

### Config Options

#### Site

    [site]

    ; enable logging of requests in redis, expiry in hours, requires geoip_country_code_by_name() to log country
    logRequestsExpiry = 8

    warnings = 0
    refresh = 0
    allowDebugFlag = 2

    ; overrides warnings/refresh/allowDebugFlag unless they are set to 2
    devMode = 0

#### Webcams

    [webcams]

    popularModelsMax = 1000
    popularModelsDays = 30

    popularModelsMaxViewersDays = 30

    popularCategoriesSnapshots = 30
    popularCategoriesRoomsWeight = 10
    popularCategoriesViewersWeight = 1

    popularCategoriesMaxViewersDays = 30

    trendingModelsSnapshots = 10
    trendingModelsMax = 1000

    ; specified in days but one snapshot of average rooms online and total viewers stored per hour
    allModelsRoomsViewersSnapshotsDays = 30

#### Users

    [users]

    alertsNumPopular = 10
    alertsNumTags = 3
    alertsPerTag = 5

#### Meta

    [meta]

    ; csv or xslx spreadsheet containing the meta
    file = meta.csv

    ; if set to true the meta rule must contain exactly all the same params/values as the requested url
    ; by default the requested urls query args must match the meta rule but the requested url can contain extra
    queryExactMatchOnly = true

    ; allows use of query args like ethnicity=white for a keyword search instead of needing to specify kwethnicity=white
    ; since kw* will perform a keyword search. meta rules should contain the requested url format ala ?ethnicity=white
    queryArgKeywordMap = ethnicity,toys
---





----


<div id='SitesFilter'/>
### Webcams Add-Ons

 `sitesFilter` - are the sites that will be  display  in `header` if add you can add `sites` or `sitesDropdown` in `headerMenuItems`
 
 &nbsp;
 &nbsp;
 &nbsp;

 `sitebarSitesFilter` - are the sites that will be  display  in  `sidebar` if add `sitesCamsDropdown` in `sidebarMenuItems`  

#### config.ini

    [webcams]
    sitesFilter                = cb,sc,cs,bc,lj,sr,cr,ff,c4,xl,xc,sm,mf,am,sp
    sitebarSitesFilter         = cb,sc,cs,bc,lj,sr,cr,ff,c4,xl,xc,sm,mf,am,sp

----

----



<div id='Promote'/>
#### Add  Website | Model | Event  - Promote

 by adding in `[urls]` -  will activate the promote Parteners functions

#### config.ini

    [urls] 
    webcamsParteners                     = [promote,          /live-webcam-model-events/:page] 
    webcamsPartenersDescription          = [promot,           /trending-webcams-live-shows/:siteNames, 'count=300'] 
    



 *****
  by adding in `[promote]` -  will make the sitemaps for promote  Website | Model | Event

    [promote]
    sitemapsUrlName                                                    = webcamsPartenersDescription
    


----


------

<div id='Add_blogs'/>
###  Blogs 

 How to Create and Use a `[blog]` Section in config.ini

#### config.ini
 Create  `URLs` for blog

    [urls] 
    articleIndex                        = [ blogs,           /blog-articles/ ] 
    articleProfile                      = [ blog,            /blog-articles/:Title/:id ]


To configure blog settings, you need to create a` [blog] `section inside your` config.ini` file. 

This section will define various settings for the blog, such as `pagination`, `publishing` `frequency`, `caching`, and `API` keys.


    [blog]
    perPage                = 36  ; Number of blog posts displayed per page
    blogPhotosUrl          = /photosBlog/  ; URL path for blog-related photos
    publishPerDay          = 1  ; One blog post will be published every 5 days (86400 seconds/day ÷ 0.2 = 432000 seconds)
    sitemapsUrlName        = articleProfile  ; Name used for sitemap URLs related to blogs
    blogGalleryUrl         = /photosBlog/GalleryImages  ; URL path for the blog's image gallery
    TinyMCEapi_key         = 'your-apikey-here'  ; API key for TinyMCE editor
    cacheExpiry            = 60  ; Cache expiration time in seconds



----


---



<div id='Add-Satelit'/>
### Satelits Sites Install

If you don’t know how to use these commands, `stop right now`. This isn’t a fucking tutorial. Screw it up, and that’s your problem.

    -A. cd /root/bin && ./add_mysql_avos.sh 
    -B. cd /root/bin && ./add_mysql_perm.sh 
    -C. su - username
    -D. cd ~/html/ && cp -Rp oldsite.com newsite.com 
    -E. cd /home/username/html/newsite.com/avos/
    -F. mysqldump -d -u oldsite -poldsite oldsite | mysql -u newsite -pnewsite newsite
    -G. cd /home/username/html/
    -H. cd newsite.com/avos && mv templates templates.safe && ln -s ../../oldsite.com/avos/templates

---

[Top](#top)
